#!/bin/bash
#
# ./migrate_exp.sh ORIGINAL_EXPID EXPID
#
# This script will clone model, templates and ocea_diagnostics
# from ORIGINAL_EXPID into EXPID git folder. It will modify the experiment 
# configuration file of the EXPID according to Autosubmit 3.x standard.
#
# Written by Domingo Manubens-Gil
#
# Institut Català de Ciències del Clima / Climate Forecasting Unit (IC3/CFU)
# Created:  November 13, 2014
copy="$1"
expid="$2"
file="/cfu/autosubmit/$expid/conf/expdef_$expid.conf"
template=$(grep TEMPLATE_NAME $file | awk '{print $3}')
mkdir /cfu/autosubmit/$expid/git
cd /cfu/autosubmit/$expid/git
git clone /cfu/autosubmit/$copy/git/model model
git clone /cfu/autosubmit/$copy/git/templates templates
git clone /cfu/autosubmit/$copy/git/ocean_diagnostics ocean_diagnostics
sed -i '/%HEADER%/d' templates/$template/*
sed -i '/%HEADER%/d' templates/common/*
sed -i '/%AS-HEADER-LOC%/d' templates/$template/*
sed -i '/%AS-HEADER-LOC%/d' templates/common/*
sed -i '/%AS-HEADER-REM%/d' templates/$template/*
sed -i '/%AS-HEADER-REM%/d' templates/common/*
sed -i '/%AS-TAILER-LOC%/d' templates/$template/*
sed -i '/%AS-TAILER-LOC%/d' templates/common/*
sed -i '/%AS-TAILER-REM%/d' templates/$template/*
sed -i '/%AS-TAILER-REM%/d' templates/common/*
echo "Unnecessary template header variables deleted..."
sed -i '/NUMPROC_SIM =*/a \
# Number of threads per task for sim jobs. NUMERIC = 1 or NUMERIC_ARRAY = 1:1:1 \
NUMTHREAD_SIM = 1 \
# Number of tasks per node for sim jobs. NUMERIC = 1 or NUMERIC_ARRAY = 24:24:24 \
NUMTASK_SIM = 1' $file
sed -i '/TEMPLATE_NAME =*/a \
# Boolean = TRUE, FALSE. Default = FALSE \
GIT_PROJECT = TRUE \
 \
#If GIT_PROJECT = FALSE no need to change \
[git] \
# type = str, help = "'"https://gitlab.cfu.local/cfu/"'" \
GIT_PROJECT_ORIGIN = '$copy' \
# type = str, default = "'"master"'", help = {"'"master"'" (defualt), "'"develop"'", ...} Check available branches here: https://gitlab.cfu.local/cfu/'$template'.git \
GIT_PROJECT_BRANCH = \
# type = str, default = leave empty, help = if model branch is a TAG leave empty.  \
GIT_PROJECT_COMMIT = \
# Where is MODEL CONFIGURATION file location relative to GIT_PROJECT_NAME root path \
GIT_FILE_MODEL_CONF = ../conf/expdef_'$expid'.conf \
# Where is HPC PLATFORM CONFIGURATION file location relative to GIT_PROJECT_NAME root path \
GIT_FILE_PLATFORM_CONF = ../conf/archdef_'$expid'.conf \
# Where is LOCALSETUP script location relative to GIT_PROJECT_NAME root path \
GIT_FILE_LOCALSETUP = templates/common/common.localsetup \
# Where is REMOTESETUP script location relative to GIT_PROJECT_NAME root path \
GIT_FILE_REMOTESETUP = templates/common/common.remotesetup \
# Where is TRANS script location relative to GIT_PROJECT_NAME root path \
GIT_FILE_TRANS = templates/common/common.localtrans \
# Where is INI script location relative to GIT_PROJECT_NAME root path \
GIT_FILE_INI = templates/'$template'/'$template'.ini \
# Where is SIM script location relative to GIT_PROJECT_NAME root path \
GIT_FILE_SIM = templates/'$template'/'$template'.sim \
# Where is POST script location relative to GIT_PROJECT_NAME root path \
GIT_FILE_POST = templates/'$template'/'$template'.post \
# Where is CLEAN script location relative to GIT_PROJECT_NAME root path \
GIT_FILE_CLEAN = templates/'$template'/'$template'.clean' $file
echo "New needed configuration variables added succesfuly..."
